@@ -4,7 +4,7 @@ from __future__ import absolute_import
 
 import os.path
 
-__version__ = '0.99.1'
+__version__ = '0.99.2'
 
 HOME = os.path.expanduser('~/.config/panucci')
 
@@ -17,4 +17,4 @@ DATABASE_FILE = os.path.join(HOME, 'panucci.sqlite')
 PLAYLIST_FILE = os.path.join(HOME, 'panucci.m3u')
 LOGFILE = os.path.join(HOME, 'panucci.log')
 
-EXTENSIONS = ('mp2', 'mp3', 'mp4', 'ogg', 'm4a', 'wav')
+EXTENSIONS = ('mp2', 'mp3', 'mp4', 'ogg', 'm4a', 'wav', 'flac')
