@@ -26,6 +26,7 @@ import random
 import panucci
 from panucci.dbsqlite import db
 from panucci.services import ObservableService
+from panucci.dbusinterface import interface
 from panucci import playlistformat
 from panucci import player
 from panucci import util
@@ -40,22 +41,27 @@ def is_supported(uri):
 class Playlist(ObservableService):
     signals = [ 'new-track-loaded', 'new-metadata-available', 'file_queued',
                 'bookmark_added', 'seek-requested', 'end-of-playlist',
-                'playlist-to-be-overwritten', 'stop-requested', 'reset-playlist' ]
+                'playlist-to-be-overwritten', 'stop-requested', 'reset-playlist',
+                'stopped', 'playing', 'paused' ]
 
     def __init__(self, config):
         self.__log = logging.getLogger('panucci.playlist.Playlist')
         ObservableService.__init__(self, self.signals, self.__log)
         self.config = config
-        
-        self.player = player.PanucciPlayer(self)
-        self.player.register( 'eof', self.on_player_eof )
-        self.__queue = Queue(None)
-        self.__queue.register(
-            'current_item_changed', self.on_queue_current_item_changed )
-
-        self.filepath = None
+        self.filepath = panucci.PLAYLIST_FILE
         self._id = None
-    
+
+        self.__player = player.PanucciPlayer(self.config)
+        self.__player.register('eof', self.on_player_eof)
+        self.__player.register('playing', self.on_player_playing)
+        self.__player.register('paused', self.on_player_paused)
+        self.__player.register('stopped', self.on_player_stopped)
+        self.__queue = Queue(self.id)
+        self.__queue.register('current_item_changed', self.on_queue_current_item_changed)
+
+        # Register the d-bus interface only once we're ready
+        interface.register_playlist(self)
+
     def init(self, filepath=None):
         """ Start playing the current file in the playlist or a custom file.
             This should be called by the UI once it has initialized.
@@ -67,16 +73,21 @@ class Playlist(ObservableService):
 
         if filepath is None or not self.load( filepath ):
             self.load_last_played()
+            position = self.get_seek_to()
+            self.set_position_duration(position, 0)
+            self.do_seek(position)
         else:
-            self.player.play()
-    
+            self.play()
+
     def reset_playlist(self):
         """ Sets the playlist to a default "known" state """
 
+        #self.notify('stop-requested', caller=self.reset_playlist)
+        self.stop()
         self.filepath = None
         self._id = None
         self.__queue.clear()
-        self.stop(None, False)
+        self.reset_player()
         self.notify('reset-playlist', caller=self.reset_playlist)
 
     @property
@@ -137,13 +148,7 @@ class Playlist(ObservableService):
         return self.save_to_new_playlist(panucci.PLAYLIST_FILE)
 
     def on_queue_current_item_changed(self):
-        if os.path.isfile(self.__queue.current_item.filepath):
-            self.notify( 'new-track-loaded',
-                     caller=self.on_queue_current_item_changed )
-            self.notify( 'new-metadata-available',
-                     caller=self.on_queue_current_item_changed )
-        else:
-            self.player.notify( "eof", caller=self.on_queue_current_item_changed )
+        self.new_track_loaded()
 
     def send_metadata(self):
         self.notify( 'new-metadata-available', caller=self.send_metadata )
@@ -151,16 +156,32 @@ class Playlist(ObservableService):
     def start_of_playlist(self):
         """Checks if the currently played item is the first"""
         if self.__queue.current_item_position == 0:
-            return True 
+            return True
 
     def end_of_playlist(self):
         """Checks if the currently played item is the last"""
         if len(self.__queue.get_items()) - 1 == self.__queue.current_item_position:
-            return True 
+            return True
+
+    def get_formatted_position(self, pos=None):
+        """ Gets the current position and converts it to a human-readable str.
+
+            Returns: (bookmark lable string, position in nanoseconds)
+        """
+
+        if pos is None:
+            (pos, dur) = self.get_position_duration()
+
+        text = util.convert_ns(pos)
+        return (text, pos)
+
+    def get_current_item(self):
+        return self.__queue.current_item
 
     def quit(self):
         self.__log.debug('quit() called.')
-        self.notify('stop-requested', caller=self.quit)
+        #self.notify('stop-requested', caller=self.quit)
+        self.stop()
         if self.__queue.modified:
             self.__log.info('Queue modified, saving temporary playlist')
             self.save_temp_playlist()
@@ -169,27 +190,31 @@ class Playlist(ObservableService):
     # Bookmark-related functions
     ######################################
 
-    def __load_from_bookmark( self, item_id, bookmark ):
+    def __load_from_bookmark(self, item_id, bookmark):
         new_pos = self.__queue.index(item_id)
         same_pos = self.__queue.current_item_position == new_pos
-        self.__queue.current_item_position = new_pos
+        if not same_pos:
+            self.__queue.current_item_position = new_pos
 
         if bookmark is None:
-            self.__queue.current_item.seek_to = 0
+            seek_position = 0
         else:
-            self.__queue.current_item.seek_to = bookmark.seek_position
+            seek_position = bookmark.seek_position
+            self.__queue.current_item.seek_to = seek_position
 
-            # if we don't request a seek nothing will happen
-            if same_pos:
-                self.notify( 'seek-requested', bookmark.seek_position,
-                    caller=self.__load_from_bookmark )
+        if same_pos:
+            if self.playing or seek_position == 0:
+                self.do_seek(seek_position)
+        if not self.playing:
+            self.play()
 
         return True
 
-    def load_from_bookmark_id( self, item_id=None, bookmark_id=None ):
+    def load_from_bookmark_id(self, item_id=None, bookmark_id=None, set_seek_to=True):
         item, bookmark = self.__queue.get_bookmark(item_id, bookmark_id)
-        if str(self.__queue.current_item) != item_id:
-            self.notify('stop-requested', caller=self.load_from_bookmark_id)
+        if str(self.__queue.current_item) != item_id and not self.null:
+            #self.notify('stop-requested', caller=self.load_from_bookmark_id)
+            self.stop(False, set_seek_to)
 
         if item is not None:
             return self.__load_from_bookmark( str(item), bookmark )
@@ -213,7 +238,7 @@ class Playlist(ObservableService):
             self.__log.info('No resume bookmark found.')
             return False
         else:
-            return self.load_from_bookmark_id( item_id, bookmark_id )
+            return self.load_from_bookmark_id(item_id, bookmark_id, False)
 
     def save_bookmark( self, bookmark_name, position ):
         if self.__queue.current_item is not None:
@@ -257,8 +282,7 @@ class Playlist(ObservableService):
         """ Updates the database entries for items that have been modified """
         for item in self.__queue:
             if item.is_modified:
-                self.__log.debug(
-                    'Playlist Item "%s" is modified, updating bookmarks', item)
+                self.__log.debug('Playlist Item "%s" is modified, updating bookmarks', item)
                 item.update_bookmarks()
                 item.is_modified = False
 
@@ -271,10 +295,16 @@ class Playlist(ObservableService):
 
         if bookmark_id is None:
             if self.__queue.current_item_position == self.__queue.index(item):
+                is_end = self.end_of_playlist()
+                self.stop()
                 item.delete_bookmark(None)
                 self.__queue.remove(item)
-                self.notify('stop-requested', caller=self.remove_bookmark)
-                self.__queue.current_item_position = self.__queue.current_item_position
+                if not is_end:
+                    self.__queue.current_item_position = self.__queue.current_item_position
+                elif not self.is_empty:
+                    self.__queue.current_item_position = 0
+                else:
+                    self.reset_playlist()
             else:
                 item.delete_bookmark(None)
                 self.__queue.remove(item)
@@ -296,6 +326,17 @@ class Playlist(ObservableService):
         else:
             return self.remove_bookmark( item_id, bookmark_id )
 
+    def add_bookmark_at_current_position( self, label=None ):
+        """ Adds a bookmark at the current position
+            Returns: (bookmark lable string, position in nanoseconds)
+        """
+
+        default_label, position = self.get_formatted_position()
+        label = default_label if label is None else label
+        self.save_bookmark( label, position )
+        self.__log.info('Added bookmark: %s - %d', label, position)
+        return label, position
+
     def move_item( self, from_row, to_row ):
         self.__log.info('Moving item from position %d to %d', from_row, to_row)
         assert isinstance(from_row, int) and isinstance(to_row, int)
@@ -374,22 +415,28 @@ class Playlist(ObservableService):
     # File importing functions
     ##################################
 
-    def load(self, filepath):
+    def load(self, filepath, play=True):
         """ Detects filepath's filetype then loads it using
             the appropriate loader function """
         self.__log.debug('Attempting to load %s', filepath)
-        _play = self.__queue.is_empty()
+        _play = self.is_empty or (self.end_of_playlist() and not self.get_position_duration()[0])
+        if self.is_empty:
+            _position = 0
+        else:
+            _position = len(self.__queue)
         error = False
 
         if os.path.isdir(filepath):
             self.load_directory(filepath, True)
         else:
-            parsers = { 'm3u': playlistformat.M3U_Playlist, 'pls': playlistformat.PLS_Playlist }
+            parsers = {'m3u': playlistformat.M3U_Playlist, 'pls': playlistformat.PLS_Playlist}
             extension = util.detect_filetype(filepath)
             if parsers.has_key(extension): # importing a playlist
                 self.__log.info('Loading playlist file (%s)', extension)
                 parser = parsers[extension](filepath, self.__queue)
-
+                self.filepath = filepath
+                self._id = None
+                self.__queue.playlist_id = self.id
                 if parser.parse(filepath):
                     self.__queue = parser.get_queue()
                     self.__file_queued( filepath, True, False )
@@ -401,23 +448,29 @@ class Playlist(ObservableService):
         # if we let the queue emit a current_item_changed signal (which will
         # happen if load_from_bookmark changes the current track), the player
         # will start playing and ingore the resume point
-        if _play:
-            self.filepath = filepath
-            self.__queue.playlist_id = self.id
-            self.__queue.disable_notifications = True
-            self.load_from_resume_bookmark()
-            self.__queue.disable_notifications = False
-            self.__queue.modified = True
-            self.notify( 'stop-requested', caller=self.load )
-            self.notify( 'new-track-loaded', caller=self.load )
-            self.notify( 'new-metadata-available', caller=self.load )
+        self.__queue.modified = True
+        if _play and not self.is_empty:
+            if not self.null:
+                self.stop(False, False)
+            if _position == 0:
+                self.__queue.disable_notifications = True
+                self.load_from_resume_bookmark()
+                self.__queue.disable_notifications = False
+            else:
+                self.__queue.set_current_item_position(_position)
+            #self.notify( 'stop-requested', caller=self.load )
+            self.new_track_loaded()
+            if play:
+                self.play()
+            #self.notify( 'new-track-loaded', caller=self.load )
+            #self.notify( 'new-metadata-available', caller=self.load )
 
         return not error
 
     def load_last_played(self):
         recent = self.get_recent_files(max_files=1)
         if recent:
-            self.load(recent[0])
+            self.load(recent[0], False)
         return bool(recent)
 
     def __file_queued(self, filepath, successfull, notify):
@@ -479,30 +532,32 @@ class Playlist(ObservableService):
     # Playlist controls
     ##################################
 
-    def set_seek_to(self, position):
+    def set_seek_to(self, seek_to, seconds=False):
         """Set the seek-to position for the current track"""
-        self.__queue.current_item.seek_to = (10**9) * position
+        if seconds:
+            seek_to = (10**9) * seek_to
+        if not self.is_empty:
+            self.get_current_item().seek_to = seek_to
 
-    def play(self):
-        """ This gets called by the player to get
-                the last time the file was paused """
-        pos = self.__queue.current_item.seek_to
-        self.__queue.current_item.seek_to = 0
-        return pos
+    def set_seek_to_from_current(self):
+        self.set_seek_to(self.get_position_duration()[0])
 
-    def pause(self, position):
-        """ Called whenever the player is paused """
-        self.__queue.current_item.seek_to = position
+    def reset_all_seek_to(self):
+        self.set_seek_to(self.get_position_duration()[0])
+        for item in self.__queue.get_items():
+            item.seek_to = 0
 
-    def stop(self, position, save_resume_point=True):
-        """ This should be run when the program is closed
-                or if the user switches playlists """
-        self.remove_resume_bookmarks()
-        if not self.is_empty and save_resume_point:
-            self.__queue.current_item.save_bookmark(
-                _('Auto Bookmark'), position, True )
+    def get_seek_to(self, reset=True):
+        """Get the seek-to position for the current track"""
+        if not self.is_empty:
+            seek_to = self.__queue.current_item.seek_to
+            if reset:
+                self.get_current_item().seek_to = 0
+        else:
+            seek_to = 0
+        return seek_to
 
-    def skip(self, loop=True, skip_by=None, skip_to=None):
+    def skip(self, loop=True, skip_by=None, skip_to=None, set_seek_to=True, play=True):
         """ Skip to another track in the playlist.
             Use either skip_by or skip_to, skip_by has precedence.
                 skip_to: skip to a known playlist position
@@ -513,10 +568,10 @@ class Playlist(ObservableService):
         if not self.__queue:
             return False
 
-        current_item = self.__queue.current_item_position
+        current_item_position = self.__queue.current_item_position
 
         if skip_by is not None:
-            skip = current_item + skip_by
+            skip = current_item_position + skip_by
         elif skip_to is not None:
             skip = skip_to
         else:
@@ -524,7 +579,7 @@ class Playlist(ObservableService):
             self.__log.warning('No skip method provided...')
 
         if not 0 <= skip < self.queue_length:
-            self.notify( 'end-of-playlist', loop, caller=self.skip )
+            #self.notify( 'end-of-playlist', loop, caller=self.skip )
 
             if not loop:
                 self.__log.warning( "Can't skip to non-existant file w/o loop."
@@ -543,65 +598,148 @@ class Playlist(ObservableService):
                 else:
                     skip = 0
 
-        self.notify('stop-requested', caller=self.skip)
+        #self.notify('stop-requested', caller=self.skip)
+        self.stop(False, set_seek_to)
         self.__queue.current_item_position = skip
-        self.__log.debug( 'Skipping to file %d (%s)', skip,
-                          self.__queue.current_item.filepath )
+        self.__log.debug('Skipping to file %d (%s)', skip, self.__queue.current_item.filepath)
+        if play:
+            self.play()
 
         return True
 
-    def get_current_item(self):
-        return self.__queue.current_item
-
-    def next(self, loop=False):
+    def next(self, loop=False, set_seek_to=True):
         """ Move the playlist to the next track.
             False indicates end of playlist. """
-        return self.skip( loop, skip_by=1)
+        return self.skip(loop, 1, None, set_seek_to)
 
-    def prev(self):
+    def prev(self, set_seek_to=True):
         """ Same as next() except moves to the previous track. """
-        return self.skip( loop=False, skip_by=-1 )
+        return self.skip(False, -1, None, set_seek_to)
 
-    def last(self):
+    def last(self, set_seek_to=True, play=True):
         """ Plays last file in queue. """
         skip_to = len(self.__queue.get_items()) - 1
-        return self.skip( False, None, skip_to )
+        return self.skip(False, None, skip_to, set_seek_to, play)
 
-    def random(self):
+    def random(self, set_seek_to=True):
         """ Plays random file in queue. """
         skip_to = random.choice(range(len(self.__queue.get_items())))
-        return self.skip( False, None, skip_to )
+        return self.skip(False, None, skip_to, set_seek_to)
+
+    ##################################
+    # Player controls
+    ##################################
+
+    def play(self):
+        """ This gets called by the player to get
+                the last time the file was paused """
+        self.__player.play()
+
+    def pause(self):
+        """ Called whenever the player is paused """
+        #position = self.get_position_duration()[0]
+        #self.__queue.current_item.seek_to = position
+        self.__player.pause()
+
+    def play_pause_toggle(self):
+        if self.current_filepath:
+            self.__player.play_pause_toggle()
+
+    def stop(self, save_resume_point=True, set_seek_to=True):
+        """ This should be run when the program is closed
+                or if the user switches playlists """
+        position, duration = self.get_position_duration()
+        self.remove_resume_bookmarks()
+        self.on_player_stopped()
+        self.__player.on_stop_requested()
+        if not self.is_empty and save_resume_point:
+            self.get_current_item().save_bookmark(_('Auto Bookmark'), position, True)
+        if not self.is_empty and set_seek_to and self.config.getboolean("options", "resume_all"):
+            self.set_seek_to(position)
 
     def on_player_eof(self):
         play_mode = self.config.get("options", "play_mode")
         if play_mode == "single":
             if not self.config.getboolean("options", "stay_at_end"):
+                self.stop(set_seek_to=False)
                 self.notify('end-of-playlist', False, caller=self.on_player_eof)
         elif play_mode == "random":
-            self.random()
+            self.random(set_seek_to=False)
         elif play_mode == "repeat":
-            self.next(True)
+            self.next(loop=True, set_seek_to=False)
         else:
             if self.end_of_playlist():
                 if not self.config.getboolean("options", "stay_at_end"):
-                   self.next(False)
+                   self.stop(set_seek_to=False)
             else:
-              self.next(False)
+                self.next(loop=False, set_seek_to=False)
 
-    def do_seek(self, seek_amount):
+    def on_player_playing(self):
+        self.__player.on_playing(self.get_seek_to())
+        self.notify("playing", caller=self.on_player_playing)
+
+    def on_player_paused(self, position, duration):
+        self.notify("paused", position, duration, caller=self.on_player_paused)
+
+    def on_player_stopped(self):
+        self.notify("stopped", caller=self.on_player_stopped)
+
+    def do_seek(self, from_beginning=None, from_current=None,  percent=None):
         resp = None
-        if not self.config.getboolean("options", "seek_back") or self.start_of_playlist() or seek_amount > 0:
-            resp = self.player.do_seek(from_current=seek_amount)
-        else:
-            pos_int, dur_int = self.player.get_position_duration()
-            if pos_int + seek_amount >= 0:
-                resp = self.player.do_seek(from_current=seek_amount)
+        if from_beginning is not None:
+            resp = self.__player.do_seek(from_beginning=from_beginning)
+        elif from_current is not None:
+            if not self.config.getboolean("options", "seek_back") or self.start_of_playlist() or from_current > 0:
+                resp = self.__player.do_seek(from_current=from_current)
             else:
-                self.prev()
-                pos_int, dur_int = self.player.get_position_duration()
-                resp = self.player.do_seek(from_beginning=dur_int+seek_amount)
+                pos_int, dur_int = self.get_position_duration()
+                if pos_int + from_current >= 0:
+                    resp = self.__player.do_seek(from_current=from_current)
+                else:
+                    self.prev()
+                    pos_int, dur_int = self.get_position_duration()
+                    resp = self.__player.do_seek(from_beginning=dur_int+from_current)
+        elif percent is not None:
+            resp = self.__player.do_seek(percent=percent)
         return resp
 
+    def get_position_duration(self):
+        return self.__player.get_position_duration()
+
+    def set_position_duration(self, position, duration):
+        self.__player.set_position_duration(position, duration)
+
+    @property
+    def playing(self):
+        return self.__player.playing
+
+    @property
+    def paused(self):
+        return self.__player.paused
+
+    @property
+    def stopped(self):
+        return self.__player.stopped
+
+    @property
+    def null(self):
+        return self.__player.null
+
+    @property
+    def seeking(self):
+        return self.__player.seeking
+
+    def new_track_loaded(self):
+        if os.path.isfile(self.current_filepath):
+            self.__player.load_media(self.current_filepath)
+            self.notify('new-track-loaded', caller=self.new_track_loaded)
+            self.notify('new-metadata-available', caller=self.new_track_loaded)
+        elif not self.end_of_playlist():
+            self.on_player_eof()
+
+    def reset_player(self):
+        self.__player.on_reset_playlist()
+
 class Queue(list, ObservableService):
     """ A Simple list of PlaylistItems """
 
@@ -777,6 +915,7 @@ class Queue(list, ObservableService):
         if not self.__prep_item(item):
             return False
 
+        item.playlist_id = self.playlist_id
         item.duplicate_id = self.__count_dupe_items(self, item)
         item.load_bookmarks()
 
