@@ -62,8 +62,8 @@ class PanucciGUI(object):
         self.create_actions()
         self.__player_tab = PlayerTab(self)
         self.__playlist_tab = qtplaylist.PlaylistTab(self, self.playlist)
-        if platform.MAEMO:
-            self.create_maemo_menus()
+        if platform.HANDSET:
+            self.create_handset_menus()
         else:
             self.create_menus()
         widget = QtGui.QWidget()
@@ -79,6 +79,8 @@ class PanucciGUI(object):
             statusTip="Add file to playlist", triggered=self.add_file_callback)
         self.action_add_folder = QtGui.QAction(QtGui.QIcon(':/images/open.png'), _("Add Folder").decode("utf-8"), self.main_window, shortcut="Ctrl+D",
             statusTip="Add folder to playlist", triggered=self.add_folder_callback)
+        self.action_play_one = QtGui.QAction(QtGui.QIcon(''), _("Play One").decode("utf-8"), self.main_window, shortcut="Ctrl+O",
+            statusTip="Play one file", triggered=self.play_one_callback)
         self.action_save_playlist = QtGui.QAction(QtGui.QIcon(':/images/save.png'), _("Save Playlist").decode("utf-8"), self.main_window, shortcut="Ctrl+W",
             statusTip="Save current playlist as m3u", triggered=self.save_playlist_callback)
         self.action_clear_playlist = QtGui.QAction(QtGui.QIcon(':/images/trashcan.png'), _("Clear Playlist").decode("utf-8"), self.main_window, shortcut="Ctrl+H",
@@ -115,6 +117,10 @@ class PanucciGUI(object):
             statusTip="Scroll title labels when too long", triggered=self.scrolling_labels_callback)
         self.action_scrolling_labels.setCheckable(True)
         self.action_scrolling_labels.setChecked(self.config.getboolean("options", "scrolling_labels"))
+        self.action_resume_all = QtGui.QAction(_("Resume All").decode("utf-8"), self.main_window, shortcut="Ctrl+R",
+            statusTip="Resume all files automatically", triggered=self.resume_all_callback)
+        self.action_resume_all.setCheckable(True)
+        self.action_resume_all.setChecked(self.config.getboolean("options", "resume_all"))
         self.action_play_mode_all = QtGui.QAction(_("All").decode("utf-8"), self.main_window, statusTip="Set play mode",
             triggered=self.play_mode_all_callback)
         self.action_play_mode_all.setCheckable(True)
@@ -150,6 +156,7 @@ class PanucciGUI(object):
         self.menu_file = self.main_window.menuBar().addMenu(_("File").decode("utf-8"))
         self.menu_file.addAction(self.action_add_file)
         self.menu_file.addAction(self.action_add_folder)
+        self.menu_file.addAction(self.action_play_one)
         self.menu_file.addAction(self.action_save_playlist)
         self.menu_file.addAction(self.action_clear_playlist)
         self.menu_file.addAction(self.action_delete_bookmarks)
@@ -167,6 +174,7 @@ class PanucciGUI(object):
         self.menu_settings.addAction(self.action_stay_at_end)
         self.menu_settings.addAction(self.action_seek_back)
         self.menu_settings.addAction(self.action_scrolling_labels)
+        self.menu_settings.addAction(self.action_resume_all)
         self.menu_play_mode = self.menu_settings.addMenu(_("Play Mode").decode("utf-8"))
         self.menu_play_mode.addAction(self.action_play_mode_all)
         self.menu_play_mode.addAction(self.action_play_mode_single)
@@ -176,13 +184,14 @@ class PanucciGUI(object):
         self.menu_help = self.main_window.menuBar().addMenu(_("Help").decode("utf-8"))
         self.menu_help.addAction(self.action_about)
 
-    def create_maemo_menus(self):
+    def create_handset_menus(self):
         # Player menu
         self.menu_player = self.main_window.menuBar().addMenu("Player")
         self.menu_player.addAction(self.action_settings)
         self.menu_player.addAction(self.action_playlist)
         self.menu_player.addAction(self.action_add_file)
         self.menu_player.addAction(self.action_add_folder)
+        self.menu_player.addAction(self.action_play_one)
         self.menu_player.addAction(self.action_clear_playlist)
         self.menu_player.addAction(self.action_timer)
         self.menu_player.addAction(self.action_about)
@@ -219,6 +228,12 @@ class PanucciGUI(object):
         if filenames:
             self._play_file(filenames[0].encode('utf-8'))
 
+    def play_one_callback(self):
+        filenames = qtutil.get_file_from_filechooser(self)
+        if filenames:
+            self.clear_playlist_callback()
+            self._play_file(filenames[0].encode('utf-8'))
+
     def save_playlist_callback(self):
         filenames = qtutil.get_file_from_filechooser(self, save_file=True, save_to=True)
         if not filenames:
@@ -251,6 +266,7 @@ class PanucciGUI(object):
                 _('By accepting all bookmarks in the database will be deleted.'), True, False, True, False)
         if response == QtGui.QMessageBox.Ok:
             self.playlist.delete_all_bookmarks()
+            self.__playlist_tab.update_model()
 
     def playlist_callback(self):
         self.__playlist_tab.main_window.show()
@@ -275,6 +291,11 @@ class PanucciGUI(object):
         self.set_config_option("scrolling_labels", str(self.action_scrolling_labels.isChecked()).lower())
         self.__player_tab.label_title.set_scrolling(self.config.getboolean("options", "scrolling_labels"))
 
+    def resume_all_callback(self):
+        self.set_config_option("resume_all", str(self.action_resume_all.isChecked()).lower())
+        if not self.action_resume_all.isChecked():
+            self.playlist.reset_all_seek_to()
+
     def play_mode_all_callback(self):
         self.set_config_option("play_mode", "all")
 
@@ -314,11 +335,11 @@ class PlayerTab(ObservableService):
         self.config = gui_root.config
         self.playlist = gui_root.playlist
         ObservableService.__init__(self, self.signals, self.__log)
+        self.metadata = None
 
-        self.playlist.player.register( 'stopped', self.on_player_stopped )
-        self.playlist.player.register( 'playing', self.on_player_playing )
-        self.playlist.player.register( 'paused', self.on_player_paused )
-        #self.playlist.player.register( 'eof', self.on_player_eof )
+        self.playlist.register( 'stopped', self.on_player_stopped )
+        self.playlist.register( 'playing', self.on_player_playing )
+        self.playlist.register( 'paused', self.on_player_paused )
         self.playlist.register( 'end-of-playlist', self.on_player_end_of_playlist )
         self.playlist.register( 'new-track-loaded', self.on_player_new_track )
         self.playlist.register( 'new-metadata-available', self.on_player_new_metadata )
@@ -333,6 +354,7 @@ class PlayerTab(ObservableService):
         layout.setSpacing(0)
         self.label_cover = QtGui.QLabel()
         self.label_cover.setContentsMargins(0, 5, 2, 5)
+        self.label_cover.mousePressEvent = self.label_cover_callback
         layout.addWidget(self.label_cover)
         vlayout = QtGui.QVBoxLayout()
         vlayout.setContentsMargins(0, 0, 0, 0)
@@ -358,34 +380,40 @@ class PlayerTab(ObservableService):
         self.progress.setFormat("00:00 / 00:00")
         self.progress.setValue(0)
         self.progress.mousePressEvent = self.on_progress_clicked
-        self.progress.setFixedHeight(self.config.getint("options", "progress_height"))
+        progress_height = self.config.getint("options", "progress_height")
+        if progress_height != -1:
+            self.progress.setFixedHeight(progress_height)
 
         self.icon_play = QtGui.QIcon(util.find_data_file('media-playback-start.png'))
         self.icon_pause = QtGui.QIcon(util.find_data_file('media-playback-pause.png'))
+
         self.button_rrewind = qtwidgets.DualActionButton(self.config,
                                                       QtGui.QIcon(util.find_data_file('media-skip-backward.png')),
                                                       self.button_rrewind_callback,
-                                         QtGui.QIcon("/usr/share/icons/gnome/24x24/actions/gtk-goto-first-ltr.png"),
+                                         QtGui.QIcon(util.find_data_file("gtk-goto-first-ltr.png")),
                                          self.playlist.prev)
-        self.button_rrewind.setFixedHeight(self.config.getint("options", "button_height"))
         self.button_rewind = QtGui.QPushButton(QtGui.QIcon(util.find_data_file('media-seek-backward.png')), "")
         self.button_rewind.clicked.connect(self.button_rewind_callback)
-        self.button_rewind.setFixedHeight(self.config.getint("options", "button_height"))
         self.button_play = QtGui.QPushButton(self.icon_play, "")
         self.button_play.clicked.connect(self.button_play_callback)
-        self.button_play.setFixedHeight(self.config.getint("options", "button_height"))
         self.button_forward = QtGui.QPushButton(QtGui.QIcon(util.find_data_file('media-seek-forward.png')), "")
         self.button_forward.clicked.connect(self.button_forward_callback)
-        self.button_forward.setFixedHeight(self.config.getint("options", "button_height"))
         self.button_fforward = qtwidgets.DualActionButton(self.config,
                                                       QtGui.QIcon(util.find_data_file('media-skip-forward.png')),
                                                       self.button_fforward_callback,
-                                         QtGui.QIcon("/usr/share/icons/gnome/24x24/actions/gtk-goto-last-ltr.png"),
+                                         QtGui.QIcon(util.find_data_file("gtk-goto-last-ltr.png")),
                                          self.playlist.next)
-        self.button_fforward.setFixedHeight(self.config.getint("options", "button_height"))
         self.button_bookmark = QtGui.QPushButton(QtGui.QIcon(util.find_data_file('bookmark-new.png')), "")
         self.button_bookmark.clicked.connect(self.button_bookmark_callback)
-        self.button_bookmark.setFixedHeight(self.config.getint("options", "button_height"))
+
+        button_height = self.config.getint("options", "button_height")
+        if button_height != -1:
+            self.button_rrewind.setFixedHeight(button_height)
+            self.button_rewind.setFixedHeight(button_height)
+            self.button_play.setFixedHeight(button_height)
+            self.button_forward.setFixedHeight(button_height)
+            self.button_fforward.setFixedHeight(button_height)
+            self.button_bookmark.setFixedHeight(button_height)
 
         layout = QtGui.QHBoxLayout()
         layout.setContentsMargins(0, 4, 0, 0)
@@ -399,23 +427,27 @@ class PlayerTab(ObservableService):
         self.mainbox.addLayout(layout)
 
         self.timer = QtCore.QTimer()
-        self.timer.setInterval(1000); 
+        self.timer.setInterval(1000)
         self.timer.timeout.connect(self.timer_callback)
 
     def on_player_stopped(self):
         self.stop_progress_timer()
-        #self.set_controls_sensitivity(False)
         self.button_play.setIcon(self.icon_play)
+        if self.metadata:
+            estimated_length = self.metadata.get('length', 0)
+            self.set_progress_callback( 0, estimated_length )
+        #self.set_controls_sensitivity(False)
 
     def on_player_playing(self):
+        self.timer_callback()
         self.start_progress_timer()
         self.button_play.setIcon(self.icon_pause)
         #self.set_controls_sensitivity(True)
 
     def on_player_paused( self, position, duration ):
         self.stop_progress_timer()
-        #self.set_progress_callback( position, duration )
         self.button_play.setIcon(self.icon_play)
+        #self.set_progress_callback( position, duration )
 
     def on_player_new_track(self):
         for widget in [self.label_title, self.label_artist, self.label_album, self.label_cover]:
@@ -426,19 +458,14 @@ class PlayerTab(ObservableService):
     def on_player_new_metadata(self):
         self.metadata = self.playlist.get_file_metadata()
         self.set_metadata(self.metadata)
-        
-        if not self.playlist.player.playing:
-            position = self.playlist.get_current_position()
-            estimated_length = self.metadata.get('length', 0)
-            self.set_progress_callback( position, estimated_length )
-            self.playlist.player.set_position_duration(position, 0)
+        position = self.playlist.get_current_position()
+        estimated_length = self.metadata.get('length', 0)
+        self.set_progress_callback(position, estimated_length)
 
     def on_player_end_of_playlist(self, loop):
         if not loop:
-            self.playlist.player.stop_end_of_playlist()
             estimated_length = self.metadata.get('length', 0)
             self.set_progress_callback( 0, estimated_length )
-            self.playlist.player.set_position_duration(0, 0)
 
     def on_player_reset_playlist(self):
         self.on_player_stopped()
@@ -456,7 +483,7 @@ class PlayerTab(ObservableService):
         self.do_seek(-1*self.config.getint("options", "seek_short"))
 
     def button_play_callback(self):
-        self.playlist.player.play_pause_toggle()
+        self.playlist.play_pause_toggle()
 
     def button_forward_callback(self):
         self.do_seek(self.config.getint("options", "seek_short"))
@@ -465,12 +492,14 @@ class PlayerTab(ObservableService):
          self.do_seek(self.config.getint("options", "seek_long"))
 
     def button_bookmark_callback(self):
-        self.playlist.player.add_bookmark_at_current_position()
+        self.playlist.add_bookmark_at_current_position()
+
+    def label_cover_callback(self, event):
+        self.playlist.play_pause_toggle()
 
     def set_progress_callback(self, time_elapsed, total_time):
         """ times must be in nanoseconds """
-        time_string = "%s / %s" % ( util.convert_ns(time_elapsed),
-            util.convert_ns(total_time) )
+        time_string = "%s / %s" %(util.convert_ns(time_elapsed), util.convert_ns(total_time))
         self.progress.setFormat( time_string )
         fraction = float(time_elapsed) / float(total_time) if total_time else 0
         self.progress.setValue( int(fraction*100) )
@@ -479,22 +508,22 @@ class PlayerTab(ObservableService):
         if ( not self.config.getboolean("options", "lock_progress") and
                 event.button() == QtCore.Qt.MouseButton.LeftButton ):
             new_fraction = float(event.x())/float(self.progress.width())
-            resp = self.playlist.player.do_seek(percent=new_fraction)
+            resp = self.playlist.do_seek(percent=new_fraction)
             if resp:
                 # Preemptively update the progressbar to make seeking smoother
                 self.set_progress_callback( *resp )
 
     def timer_callback( self ):
-        if self.playlist.player.playing and not self.playlist.player.seeking:
-            pos_int, dur_int = self.playlist.player.get_position_duration()
+        if self.playlist.playing and not self.playlist.seeking:
+            pos_int, dur_int = self.playlist.get_position_duration()
             # This prevents bogus values from being set while seeking
-            if ( pos_int > 10**9 ) and ( dur_int > 10**9 ):
+            if pos_int >= 0 and dur_int >= 0:
                 self.set_progress_callback( pos_int, dur_int )
         return True
 
     def start_progress_timer( self ):
         self.timer.start()
-        
+
     def stop_progress_timer( self ):
         self.timer.stop()
 
@@ -535,14 +564,14 @@ class PlayerTab(ObservableService):
             if tags.has_key(tag) and value is not None and value.strip():
                 value = value.decode('utf-8')
                 if tag == "artist":
-                    _str = '<big>' + cgi.escape(value) + '</big>'
+                    _str = '<big>' + value + '</big>'
                 elif tag == "album":
-                    _str = cgi.escape(value)
+                    _str = value
                 elif tag == "title":
                     _str = '<b><big>' + cgi.escape(value) + '</big></b>'
-                    if not platform.MAEMO:
+                    if not platform.HANDSET:
                         value += ' - Panucci'
-                    if platform.FREMANTLE and len(value) > 25:
+                    elif len(value) > 25:
                         value = value[:24] + '...'
                     self.__gui_root.main_window.setWindowTitle(value)
  
@@ -558,7 +587,7 @@ class PlayerTab(ObservableService):
                 tags[tag].show()
 
     def do_seek(self, seek_amount):
-        resp = self.playlist.do_seek(seek_amount*10**9)
+        resp = self.playlist.do_seek(from_current=seek_amount*10**9)
         if resp:
             # Preemptively update the progressbar to make seeking smoother
             self.set_progress_callback( *resp )
