@@ -1,4 +1,4 @@
-#!/usr/bin/python
+# -*- coding: utf-8 -*-
 #
 # This file is part of Panucci.
 # Copyright (c) 2008-2011 The Panucci Project
@@ -15,36 +15,39 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Panucci.  If not, see <http://www.gnu.org/licenses/>.
-#
 
 from __future__ import absolute_import
 
 import logging
+import os.path
+import ConfigParser
 
 import panucci
+from panucci import util
 
-import os.path
-
-LAST_FOLDER_DEFAULT = os.path.join(os.path.expanduser('~'), 'MyDocs')
-
-if not os.path.exists(LAST_FOLDER_DEFAULT):
-    LAST_FOLDER_DEFAULT = os.path.expanduser('~')
-
-DEFAULTS = {
-    'dual_action_button_delay'  : 0.5,
-    'enable_dual_action_btn'    : True,
-    'last_folder'               : LAST_FOLDER_DEFAULT,
-    'max_recent_files'          : 10,
-    'progress_locked'           : False,
-    'scrolling_labels'          : True,
-    'seek_long'                 : 60,
-    'seek_short'                : 10,
-}
+if not os.path.exists(panucci.SETTINGS_FILE):
+    import shutil
+    _filepath = util.find_data_file("panucci.conf")
+    shutil.copy(_filepath, panucci.HOME)
 
 class Settings(object):
     def __init__(self):
         self.__log = logging.getLogger('panucci.settings.Settings')
-
+        self.config = ConfigParser.SafeConfigParser()
+        # Parse everything
+        _file = open(util.find_data_file("panucci-all.conf"))
+        self.config.readfp(_file)
+        _file.close()
+        # Parse non editable 
+        if os.path.exists(panucci.HOME + '/panucci-noedit.conf'):
+            _file = open(panucci.HOME + "/panucci-noedit.conf")
+            self.config.readfp(_file)
+            _file.close()
+        # Parse editable
+        _file = open(panucci.SETTINGS_FILE)
+        self.config.readfp(_file)
+        _file.close()
+    """
     def __getattr__(self, key):
         if DEFAULTS.has_key(key):
             # FIXME: Load settings from somewhere
@@ -78,6 +81,5 @@ class Settings(object):
         #self.register(
         #    setting + SIGNAL_NAME_SUFFIX, lambda v: button.set_active(v) )
         button.set_active( getattr(self, setting) )
-
+     """
 settings = Settings()
-
